@@ -16,7 +16,17 @@ class Ignite:
     the event handler system of PyTorch Ignite.
     """
 
-    def __init__(self, adapter, logger=None, log_freq=50, with_pbars=True):
+    def __init__(
+        self,
+        adapter,
+        validator=None,
+        stat_getter=None,
+        saver=None,
+        logger=None,
+        log_freq=50,
+        with_pbars=True,
+        device=None,
+    ):
         """
         Arguments:
             adapter: An [Adapter](../../adapters/index.md) object
@@ -26,36 +36,30 @@ class Ignite:
                 each epoch.
         """
         self.adapter = adapter
+        self.validator = validator
+        self.stat_getter = stat_getter
+        self.saver = saver
         self.logger = c_f.default(logger, IgniteEmptyLogger, {})
         self.log_freq = log_freq
         self.with_pbars = with_pbars
-        self.engine_init()
+        self.device = c_f.default(device, idist.device, {})
+        self.trainer_init()
+        self.collector_init()
         self.dist_init_done = False
-        self.dist_init()
+        if device is None:
+            self.dist_init()
+        self.temp_events = []
 
     def training_step(self, engine, batch):
-        device = idist.device()
-        batch = c_f.batch_to_device(batch, device)
-        return self.adapter.training_step(batch, device, self)
+        batch = c_f.batch_to_device(batch, self.device)
+        return self.adapter.training_step(batch, self)
 
     def before_training_starts(self, engine):
         self.adapter.before_training_starts(self)
 
-    def engine_init(self):
+    def trainer_init(self):
         self.trainer = Engine(self.training_step)
         self.trainer.state.adapter = self.adapter
-        self.labeled_collector = Engine(
-            self.get_labeled_collector_step(self.adapter.inference)
-        )
-        self.unlabeled_collector = Engine(
-            self.get_unlabeled_collector_step(self.adapter.inference)
-        )
-        i_g.do_for_all_engines(self, i_g.set_engine_logger)
-        self.register_event_handlers()
-
-    def register_event_handlers(self):
-        if self.with_pbars:
-            pbars = i_g.do_for_all_engines(self, i_g.attach_pbar)
         i_g.register(self.trainer, Events.STARTED, self.before_training_starts)
         i_g.register(
             self.trainer, Events.EPOCH_STARTED, self.set_to_train(self.adapter.models)
@@ -64,6 +68,7 @@ class Ignite:
             i_g.step_lr_schedulers(self.adapter.lr_schedulers, "per_step"),
             TerminateOnNan(),
         ]
+        pbars = i_g.set_loggers_and_pbars(self, ["trainer"])
         if self.with_pbars:
             iteration_complete.append(i_g.pbar_print_losses(pbars["trainer"]))
         i_g.register(self.trainer, Events.ITERATION_COMPLETED, *iteration_complete)
@@ -79,6 +84,15 @@ class Ignite:
             self.logger.write,
             i_g.zero_grad(self.adapter),
         )
+
+    def collector_init(self):
+        self.labeled_collector = Engine(
+            self.get_labeled_collector_step(self.adapter.inference)
+        )
+        self.unlabeled_collector = Engine(
+            self.get_unlabeled_collector_step(self.adapter.inference)
+        )
+        i_g.set_loggers_and_pbars(self, ["labeled_collector", "unlabeled_collector"])
         i_g.register(
             self.labeled_collector,
             Events.EPOCH_STARTED,
@@ -119,11 +133,8 @@ class Ignite:
         self,
         datasets,
         dataloader_creator=None,
-        validator=None,
-        stat_getter=None,
         validation_interval=1,
         patience=10,
-        saver=None,
         resume=None,
         check_initial_accuracy=False,
         **trainer_kwargs,
@@ -131,54 +142,75 @@ class Ignite:
         dataloader_creator = c_f.default(dataloader_creator, DataloaderCreator())
         dataloaders = dataloader_creator(**datasets)
 
-        if validator:
+        self.remove_temp_events()
+
+        if self.validator:
             max_epochs = trainer_kwargs.get("max_epochs", 1)
-            i_g.add_validation_runner(
-                self,
+            self.add_validation_runner(
                 dataloaders,
-                validator,
-                stat_getter,
                 validation_interval,
                 max_epochs,
-                saver,
-                self.logger,
                 check_initial_accuracy,
             )
 
-            self.trainer.add_event_handler(
-                Events.EPOCH_STARTED, i_g.early_stopper(patience, validator)
+            self.add_temp_event_handler(
+                Events.EPOCH_STARTED, i_g.early_stopper(patience, self.validator)
             )
 
-        if saver:
-            if not validator:
-                self.trainer.add_event_handler(
+        if self.saver:
+            if not self.validator:
+                self.add_temp_event_handler(
                     Events.EPOCH_COMPLETED,
-                    i_g.save_adapter_without_validator(saver, self.adapter),
+                    i_g.save_adapter_without_validator(self.saver, self.adapter),
                 )
-            self.trainer.add_event_handler(Events.EPOCH_COMPLETED, saver.save_ignite)
+            self.add_temp_event_handler(Events.EPOCH_COMPLETED, self.saver.save_ignite)
 
         if resume is not None:
             if resume != "latest":
                 raise ValueError("Only 'latest' resume is currently supported")
-            if not saver:
+            if not self.saver:
                 raise ValueError("To resume, a Saver must be provided")
-            saver.load_all(
+            self.saver.load_all(
                 adapter=self.adapter,
-                validator=validator,
-                stat_getter=stat_getter,
+                validator=self.validator,
+                stat_getter=self.stat_getter,
                 framework=self,
                 suffix=resume,
             )
-            i_g.resume_checks(validator, stat_getter, self)
+            i_g.resume_checks(self.validator, self.stat_getter, self)
 
         if not i_g.is_done(self.trainer, **trainer_kwargs):
             self.trainer.run(dataloaders["train"], **trainer_kwargs)
 
-        if validator:
-            return validator.best_score, validator.best_epoch
+        if self.validator:
+            return self.validator.best_score, self.validator.best_epoch
 
         return None, None
 
+    def add_validation_runner(
+        self,
+        dataloaders,
+        validation_interval,
+        max_epochs,
+        check_initial_accuracy,
+    ):
+        validation_condition = Events.EPOCH_COMPLETED(every=validation_interval)
+        if max_epochs % validation_interval != 0:
+            validation_condition |= Events.EPOCH_COMPLETED(once=max_epochs)
+        if check_initial_accuracy:
+            validation_condition |= Events.STARTED
+        self.add_temp_event_handler(
+            validation_condition,
+            i_g.get_validation_runner(
+                self,
+                dataloaders,
+                self.validator,
+                self.stat_getter,
+                self.saver,
+                self.logger,
+            ),
+        )
+
     def evaluate_best_model(
         self, datasets, validator, saver, epoch, dataloader_creator=None
     ):
@@ -200,9 +232,8 @@ class Ignite:
 
     def get_x_collector_step(self, inference, name):
         def collector_step(engine, batch):
-            device = idist.device()
             with torch.no_grad():
-                batch = c_f.batch_to_device(batch, device)
+                batch = c_f.batch_to_device(batch, self.device)
                 features, logits = inference(
                     batch[f"{name}_imgs"], domain=batch[f"{name}_domain"]
                 )
@@ -242,3 +273,12 @@ class Ignite:
             models.eval()
 
         return handler
+
+    def add_temp_event_handler(self, event, handler):
+        self.trainer.add_event_handler(event, handler)
+        self.temp_events.append((handler, event))
+
+    def remove_temp_events(self):
+        for h, e in self.temp_events:
+            self.trainer.remove_event_handler(h, e)
+        self.temp_events = []
