@@ -2,7 +2,6 @@ import logging
 
 import ignite.distributed as idist
 from ignite.contrib.handlers import ProgressBar
-from ignite.engine import Events
 from ignite.utils import setup_logger
 
 from ...utils import common_functions as c_f
@@ -11,35 +10,6 @@ from ...weighters import get_multiple_loss_totals
 from .dictionary_accumulator import DictionaryAccumulator
 
 
-def add_validation_runner(
-    ignite_wrapper,
-    dataloaders,
-    validator,
-    stat_getter,
-    validation_interval,
-    max_epochs,
-    saver,
-    logger,
-    check_initial_accuracy,
-):
-    validation_condition = Events.EPOCH_COMPLETED(every=validation_interval)
-    if max_epochs % validation_interval != 0:
-        validation_condition |= Events.EPOCH_COMPLETED(once=max_epochs)
-    if check_initial_accuracy:
-        validation_condition |= Events.STARTED
-    ignite_wrapper.trainer.add_event_handler(
-        validation_condition,
-        get_validation_runner(
-            ignite_wrapper,
-            dataloaders,
-            validator,
-            stat_getter,
-            saver,
-            logger,
-        ),
-    )
-
-
 def get_validation_runner(
     cls,
     dataloaders,
@@ -182,13 +152,19 @@ def register(engine, event, *args):
         engine.add_event_handler(event, h)
 
 
-def do_for_all_engines(cls, function):
+def do_for_all_engines(cls, function, keys):
     output = {}
-    for name in ["trainer", "labeled_collector", "unlabeled_collector"]:
+    for name in keys:
         output[name] = function(getattr(cls, name), name)
     return output
 
 
+def set_loggers_and_pbars(cls, keys):
+    do_for_all_engines(cls, set_engine_logger, keys)
+    if cls.with_pbars:
+        return do_for_all_engines(cls, attach_pbar, keys)
+
+
 def resume_checks(validator, stat_getter, framework):
     last_trainer_epoch = framework.trainer.state.epoch
     for name, v in {"validator": validator, "stat_getter": stat_getter}.items():
